!function(global, factory) {
    'object' == typeof exports && 'undefined' != typeof module ? factory(exports) : 'function' == typeof define && define.amd ? define([ 'exports' ], factory) : factory(global.typedash = global.typedash || {});
}(this, function(exports) {
    'use strict';
    function curry(f, n) {
        var argsArray = argumentsArray(arguments);
        if (void 0 === n) argsArray[1] = f.length;
        if (n === argsArray.length - 2) return f.apply(void 0, argsArray.slice(2)); else return function() {
            return curry.apply(void 0, concat(argsArray, argumentsArray(arguments)));
        };
    }
    function noop() {}
    function identity(i) {
        return i;
    }
    function debounce(func, wait, immediate) {
        if (void 0 === immediate) immediate = !1;
        var timeout;
        return function() {
            var context = this;
            var args = arguments;
            var later = function() {
                timeout = null;
                if (!immediate) func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
            if (callNow) func.apply(context, args);
        };
    }
    function once(func) {
        var called = !1;
        var res;
        return function() {
            if (!called) {
                called = !0;
                res = func.apply(this, arguments);
            }
            return res;
        };
    }
    function memoize(func) {
        var stringifyJson = JSON.stringify;
        var cache = {};
        return function() {
            var hash = stringifyJson(arguments);
            return hash in cache ? cache[hash] : cache[hash] = func.apply(this, arguments);
        };
    }
    function delayed(schedulerFn, fn) {
        function handle() {
            fn.apply(ctx, lastArgs);
            ticking = !1;
        }
        var ticking = !1;
        var ctx = this;
        var lastArgs = [];
        return function() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) args[_i] = arguments[_i];
            if (!ticking) schedulerFn(handle);
            ticking = !0;
            lastArgs = args;
            ctx = this;
        };
    }
    function filter(array, callbackfn) {
        return array.filter(callbackfn);
    }
    function forEach(array, callbackfn) {
        array.forEach(callbackfn);
        return array;
    }
    function indexOf(array, item) {
        return array.indexOf(item);
    }
    function compact(array) {
        return filter(array, identity);
    }
    function argumentsArray(argsObject) {
        return arrayPrototype.slice.call(argsObject, 0);
    }
    function concat() {
        var item = [];
        for (var _i = 0; _i < arguments.length; _i++) item[_i] = arguments[_i];
        return arrayPrototype.concat.apply([], item);
    }
    function flatten(array) {
        return concat.apply(void 0, [ [] ].concat(array));
    }
    function uniq(array) {
        return filter(array, function(item, index, array) {
            return indexOf(array, item) === index;
        });
    }
    function without(array, remove) {
        return filter(array, function(item) {
            return !contains(remove, item);
        });
    }
    function contains(array, item) {
        return -1 !== indexOf(array, item);
    }
    function difference(array1, array2) {
        return concat(without(array1, array2), without(array2, array1));
    }
    function isArray(item) {
        return Array.isArray(item);
    }
    function keys(item) {
        return Object.keys(item);
    }
    function isEmpty(item) {
        return !item || isArray(item) && !item.length || !keys(item).length;
    }
    function values(item) {
        return keys(item).reduce(function(res, key) {
            return concat(res, item[key]);
        }, []);
    }
    function mapKeyValues(item, mapKeyFn, mapValueFn) {
        return keys(item).reduce(function(res, key) {
            res[mapKeyFn(key)] = mapValueFn(item[key]);
            return res;
        }, {});
    }
    function mapKeys(item, mapKeyFn) {
        return mapKeyValues(item, mapKeyFn, identity);
    }
    function mapValues(item, mapValueFn) {
        return mapKeyValues(item, identity, mapValueFn);
    }
    function reduce(item, reduceFn) {
        return keys(item).reduce(function(res, key) {
            res[key] = reduceFn(key, item[key]);
            return res;
        }, {});
    }
    function filter$1(item, filterFn) {
        return keys(item).reduce(function(res, key) {
            if (filterFn(key)) res[key] = item[key];
            return res;
        }, {});
    }
    function omit(item, keys) {
        return filter$1(item, function(key) {
            return !contains(keys, key);
        });
    }
    function pick(item, keys) {
        return filter$1(item, function(key) {
            return contains(keys, key);
        });
    }
    function isMergeableObject(val) {
        return val && 'object' == typeof val && '[object RegExp]' !== toString.call(val) && '[object Date]' !== toString.call(val);
    }
    function emptyTarget(val) {
        return isArray(val) ? [] : {};
    }
    function cloneIfNecessary(value) {
        return isMergeableObject(value) ? merge(emptyTarget(value), value) : value;
    }
    function arrayMerge(target, source) {
        var destination = target.slice();
        forEach(source, function(e, i) {
            if (void 0 === destination[i]) destination[i] = cloneIfNecessary(e); else if (isMergeableObject(e)) destination[i] = merge(target[i], e); else if (-1 === target.indexOf(e)) destination.push(cloneIfNecessary(e));
        });
        return destination;
    }
    function mergeObject(target, source) {
        var destination = {};
        if (isMergeableObject(target)) forEach(keys(target), function(key) {
            destination[key] = cloneIfNecessary(target[key]);
        });
        forEach(keys(source), function(key) {
            if (!isMergeableObject(source[key]) || !target[key]) destination[key] = cloneIfNecessary(source[key]); else destination[key] = merge(target[key], source[key]);
        });
        return destination;
    }
    function merge(target, source) {
        if (isArray(source)) return isArray(target) ? arrayMerge(target, source) : cloneIfNecessary(source); else return mergeObject(target, source);
    }
    function patch(old, patch) {
        return filter$1(patch, function(key) {
            return patch[key] !== old[key];
        });
    }
    function shallowEqual(objA, objB) {
        if (objA === objB) return !0;
        if ('object' != typeof objA || !objA || 'object' != typeof objB || !objB) return !1;
        var keysA = keys(objA);
        var keysB = keys(objB);
        if (keysA.length !== keysB.length) return !1;
        var bHasOwnProperty = Object.prototype.hasOwnProperty.bind(objB);
        for (var idx = 0; idx < keysA.length; idx++) {
            var key = keysA[idx];
            if (!bHasOwnProperty(key)) return !1;
            var valueA = objA[key];
            var valueB = objB[key];
            if (valueA !== valueB) return !1;
        }
        return !0;
    }
    function words(str) {
        return str && str.match(asciiWords) || [];
    }
    function mapWords(str, wordMapFn, join) {
        if (void 0 === join) join = '';
        return words(str).map(wordMapFn).join(join);
    }
    function lowerCase(s) {
        return s.toLowerCase();
    }
    function caseStr(casing, str) {
        return mapWords(str, lowerCase, casing);
    }
    function capitalize(str) {
        return str[0].toUpperCase() + str.slice(1);
    }
    function camelCase(str) {
        return mapWords(str, function(word, i) {
            word = lowerCase(word);
            return 0 === i ? word : capitalize(word);
        });
    }
    function trim(str) {
        return str.trim();
    }
    function wildcardToRegExp(wildcardPattern, flags) {
        var regexMatch = /^\/(.+)\/([im]+)?$/.exec(wildcardPattern);
        if (regexMatch) return new RegExp(regexMatch[1], regexMatch[2] || flags);
        var pattern = wildcardPattern.replace(/[\-\[\]\/{}()?.\\^$|]/g, '\\$&').replace(/\*/g, '.*?').replace(/\+/g, '.+?');
        return new RegExp("^" + pattern + "$", flags);
    }
    function template(template) {
        var sanitized = template.split(/(\${([\s]*[^;\s{]+[\s]*)})/g);
        var fn = 'return [';
        for (var i = 0; i < sanitized.length; i++) if ('$' === sanitized[i][0]) fn += "$d('" + sanitized[++i] + "'),"; else fn += "\"" + sanitized[i].replace('"', '\\"') + "\",";
        return Function('$d', fn + "]");
    }
    function templateObject(value, nullValue) {
        return function(key) {
            return value[key] || nullValue;
        };
    }
    var arrayPrototype = Array.prototype;
    var toString = Object.prototype.toString;
    var asciiWords = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
    var kebabCase = curry(caseStr)('-');
    var snakeCase = curry(caseStr)('_');
    exports.forEach = forEach;
    exports.indexOf = indexOf;
    exports.compact = compact;
    exports.argumentsArray = argumentsArray;
    exports.concat = concat;
    exports.flatten = flatten;
    exports.uniq = uniq;
    exports.without = without;
    exports.contains = contains;
    exports.difference = difference;
    exports.isArray = isArray;
    exports.keys = keys;
    exports.isEmpty = isEmpty;
    exports.values = values;
    exports.mapKeyValues = mapKeyValues;
    exports.mapKeys = mapKeys;
    exports.mapValues = mapValues;
    exports.reduce = reduce;
    exports.filter = filter$1;
    exports.omit = omit;
    exports.pick = pick;
    exports.merge = merge;
    exports.patch = patch;
    exports.shallowEqual = shallowEqual;
    exports.curry = curry;
    exports.noop = noop;
    exports.identity = identity;
    exports.debounce = debounce;
    exports.once = once;
    exports.memoize = memoize;
    exports.delayed = delayed;
    exports.words = words;
    exports.lowerCase = lowerCase;
    exports.kebabCase = kebabCase;
    exports.snakeCase = snakeCase;
    exports.capitalize = capitalize;
    exports.camelCase = camelCase;
    exports.trim = trim;
    exports.wildcardToRegExp = wildcardToRegExp;
    exports.template = template;
    exports.templateObject = templateObject;
    Object.defineProperty(exports, '__esModule', {
        value: !0
    });
});
//# sourceMappingURL=typedash.js.map