/***
 * Iterate over array with `callbackfn`
 * @param array
 * @param callbackfn
 * @returns {T[]}
 */
export declare function forEach<T>(array: T[], callbackfn: (this: void, value: T, index: number, array: T[]) => any): T[];
/***
 * Get index of item in array
 * @param array
 * @param item
 * @returns {number} -1 if item is not found
 */
export declare function indexOf<T>(array: T[], item: T): number;
/***
 * Creates an array with all falsey values removed.
 * The values `false`, `null`, `0`, `""`, `undefined`, and `NaN` are falsey.
 * @param array array to compact.
 * @returns {T[]} Returns the new array of filtered values.
 */
export declare function compact<T>(array: (T | null | undefined)[]): T[];
/***
 * Return array from arguments
 * @param argsObject
 * @returns {any[]}
 */
export declare function argumentsArray<T>(argsObject: IArguments): T[];
/***
 * Creates a new array concatenating array with any additional arrays and/or values.
 * @param item item or array to concatenate
 * @returns {T[]} Returns the new concatenated array.
 */
export declare function concat<T>(...item: (T | T[])[]): T[];
/***
 * Flattens array a single level deep.
 * @param array The array to flatten.
 * @returns {T[]} Return Flattened array.
 */
export declare function flatten<T>(array: (T | T[])[]): T[];
/***
 * Creates a duplicate-free version of an array,
 * using `indexOf` for equality comparisons,
 * in which only the first occurrence of each element is kept.
 * The order of result values is determined by the order they occur in the array.
 * @param array The array to inspect.
 * @returns {T[]} Duplicate free array
 */
export declare function uniq<T>(array: T[]): T[];
/***
 * Creates an array excluding all given values using `indexOf` for equality comparisons.
 * @param array The array to inspect.
 * @param remove The array to exclude.
 * @returns {T[]} Returns the new array of filtered values.
 */
export declare function without<T>(array: T[], remove: T[]): T[];
/***
 * Check if item is in array using `indexOf`
 * @param array The array to inspect
 * @param item The item to check
 * @returns {boolean} Return `true` if item is in array
 */
export declare function contains<T>(array: T[], item: T): boolean;
/***
 * Creates an symmetric difference array of values not included in the other given arrays.
 * The order and references of result values are determined by the first array.
 * @param array1 The first array
 * @param array2 The second array
 * @returns {[T]} Returns values not included in the other given arrays.
 */
export declare function difference<T>(array1: T[], array2: T[]): T[];
/***
 * Check if argument is an array
 * @param item
 * @returns {boolean}
 */
export declare function isArray<T>(item: T | T[]): item is Array<T>;
