import { CurriedFunction1, CurriedFunction2, CurriedFunction3 } from './typedash';
/***
 * Creates a function that accepts arguments of func and either invokes func returning its result,
 * if at least arity number of arguments have been provided, or returns a function that accepts the remaining func arguments,
 * and so on. The arity of func may be specified if func.length is not sufficient.
 * @param fn
 * @param n
 */
export declare function curry<T1, TResult>(fn: (a: T1) => TResult, n?: number): CurriedFunction1<T1, TResult>;
export declare function curry<T1, T2, TResult>(fn: (a: T1, b: T2) => TResult, n?: number): CurriedFunction2<T1, T2, TResult>;
export declare function curry<T1, T2, T3, TResult>(fn: (a: T1, b: T2, c: T3) => TResult, n?: number): CurriedFunction3<T1, T2, T3, TResult>;
/***
 * Function that do nothing
 */
export declare function noop(): void;
/***
 * Function for identity enumeration. Returns what passed to it
 * @param i
 * @returns {T}
 */
export declare function identity<T>(i: T): T;
/***
 *Creates a debounced function that delays invoking func until after wait milliseconds have elapsed
 * since the last time the debounced function was invoked.
 * The func is invoked with the last arguments provided to the debounced function.
 * @param func
 * @param wait
 * @param immediate indicate whether func should be invoked on the leading and/or trailing edge of the wait timeout.
 * @returns {Function}
 */
export declare function debounce<T extends Function>(func: T, wait: number, immediate?: boolean): T;
/***
 * Creates a function that is restricted to invoking func once.
 * Repeat calls to the function return the value of the first invocation.
 * The func is invoked with the this binding and arguments of the created function.
 * @param func
 * @returns {T}
 */
export declare function once<T extends Function>(func: T): T;
/***
 * Creates a function that memoizes the result of func.
 * The func is invoked with the this binding of the memoized function.
 * @param func
 * @returns {T}
 */
export declare function memoize<T extends Function>(this: any, func: T): T;
/***
 * Creates a delayed function that delays invoking func until called by schedulerFn.
 * The func is invoked with the last arguments provided to the delayed function.
 * @param schedulerFn
 * @param fn
 * @returns {T}
 */
export declare function delayed<T extends Function>(this: any, schedulerFn: (fn: Function) => void, fn: T): T;
