/***
 * Spit string into words, unicode is not supported
 * @param str The string to split
 * @returns {RegExpMatchArray|[]} Matched words
 */
export declare function words(str?: string): string[];
/***
 * Convert string to lowerCase
 * @param s The string to convert
 * @returns {string} Returns string in lowercase
 */
export declare function lowerCase(s: string): string;
/***
 * Convert string to kebab-case
 * @param s The string to convert
 * @returns {string} Returns string in kebab-case
 */
export declare const kebabCase: (v2: string) => string;
/***
 * Convert string to snake_case
 * @param s The string to convert
 * @returns {string} Returns string in snake_case
 */
export declare const snakeCase: (v2: string) => string;
/***
 * Capitalize first char of string
 * @param str
 * @returns {string} Returns string with first char capitalized
 */
export declare function capitalize(str: string): string;
/***
 * Convert string to camelCase
 * @param str The string to convert
 * @returns {string} Returns string in camelCase
 */
export declare function camelCase(str: string): string;
/***
 * Removes leading and trailing whitespace or specified characters from string.
 * @param str The string to trim.
 * @returns {string} Trimmed string
 */
export declare function trim(str: string): string;
/***
 * Convert wildcard string to RegExp. Like `hel*o` becames `/hel.*o/`
 * @param wildcardPattern pattern to convert
 * @param flags flags for created regex 'igm'
 * @returns {RegExp} Constructed regexp
 */
export declare function wildcardToRegExp(wildcardPattern: string, flags?: string): RegExp;
/***
 * Compile string template into template function.
 * Template language is similar to template strings in es6 `Hello ${name}`.
 * Doesn't support anything but property name inside `${}`
 * @example Use `template('Hello ${name}')(templateObject({name:'Bob'}))==='Hello Bob'`
 * @param template The template string
 * @returns {(data:<T>(key: keyof T) => string)=>string} Compiled template function.
 */
export declare function template<T>(template: string): (data: <T>(key: keyof T) => T) => (string | T)[];
/***
 * Function to use with `template` return object keys by template key
 * @param value
 * @param nullValue
 * @returns {(key:keyof T)=>string?}
 */
export declare function templateObject<T>(value: {
    [key: string]: T;
}, nullValue?: T): (key: string) => T;
